package hsmw.jotto5.beleg.views;

import java.io.FileNotFoundException;
import com.sun.net.httpserver.*;
import java.io.FileInputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.io.File;

/**
 * Fängt alle Anfragen an "/" und unbekannte URLs ab.
 *
 * Ist die View die aufgerufen wird wenn die Basis-Domain der Anwendung 
 * aufgerufen wird. Aufgrund des eigenartigen Routings des HttpServers
 * übernimmt diese Klasse auch ein Paar sonstige Routing-Aufgaben, wie das
 * Anzeigen der 404-Seite bei nicht existierendem Endpunkt.
 */
public class RootView implements HttpHandler {

	private void handleFileOr404(HttpExchange t) throws IOException {
		File f;
		FileInputStream fis;
		OutputStream os;
		int len;
		byte[] buf = new byte[4096]; // Puffer um die Datei einzulesen
		
		f = new File("static" + t.getRequestURI().getPath());

		fis = null;
		os = t.getResponseBody();
		try {
			fis = new FileInputStream(f);
			// siehe https://www.baeldung.com/java-inputstream-to-outputstream
			t.sendResponseHeaders(200, f.length());
			while ((len = fis.read(buf)) != -1)
				os.write(buf, 0, len);
		} catch (FileNotFoundException e) {
			// TODO: proper 404 response
			// Datei nicht gefunden, stacktrace also unnötig
			t.sendResponseHeaders(404, 0);	
		} catch (Exception e) {
			// TODO: proper 500 response
			t.sendResponseHeaders(500, 0);
			e.printStackTrace();
		} finally {
			if (fis != null) fis.close();
			os.close();
		}
	}

	public void handle(HttpExchange t) throws IOException {
		// prüfen, ob WIRKLICH der Root-Endpunkt gemeint ist
		if (t.getRequestURI().getPath().equals("/")) {
			// wir leiten an /main weiter, dort passiert dann die restliche Magie
			t.getResponseHeaders().add("Location", "/main");
			t.sendResponseHeaders(308, 0); // 308 => "Permanent Redirect"
			t.getResponseBody().close();
		} else {
			// entweder eine statische Datei oder ein ungültiger Pfad
			handleFileOr404(t);
		}
	}

}
